#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class CheckDruidHistoricalJob < Struct.new(:user_id, :verbose)
      include MonitorHelper
      include AlertsHelper
      include NotificationsHelper

      def perform
        content = ""
        content << "Node executing this job: #{`hostname`.chomp!}\n\n"
        @status = true
        @next_run = Time.current + 5.minutes
        begin
          content << checkDruidHistoricals
        rescue Exception => e
          content << "#{e.class}: #{e.message}\n"
          e.backtrace.each do |line|
            content << "#{line}\n"
          end
        ensure
          # Save as StoredDelayedJob only if it has an ERROR
          StoredDelayedJob.create(job: 'CheckDruidHistoricalJob', user_id: user_id, content: content) unless @status
          Delayed::Job.enqueue(self.class.new(user_id, true), priority: 10, run_at: @next_run)
        end
      end

      private

      ###
      # Get information of Druid Historicals
      ###    
      def checkDruidHistoricals
        logs = ""
        sep = "\n-----------------------------------------------------------\n"

        managers = Manager.all.sort_by(&:name)
        hosts = managers.first.node['redborder']['managers_per_services']['druid-historical']
        cluster = Redborder::Cluster.instance

        errors = druid_historical_get_all_errors(hosts, cluster)
        alert_params = {
          message: "#{I18n.t('lib_redborder.errors_druid.notification_msg')}"
        }
        if errors.values.any? { |v| !v.nil? && !v.empty? }
          # Steps when Druid Historical has an ERROR or WARNING
          @status = false

          logs << "#{I18n.t('lib_redborder.errors_druid.header')}"
          logs << sep

          logs << show_errors(errors)
          logs << sep

          logs << "More information:\n"
          logs << "\t#{I18n.t('lib_redborder.errors_druid.next_run')}" + " '#{@next_run.to_s}'\n"

          stop_alert_with_params(alert_params)
          max_error_level = check_errors(errors)
          if max_error_level != 'ok'
            max_error_level = 'danger' if max_error_level == 'error'
            alert_params[:level] = max_error_level

            logs << create_druid_historical_notification

            unless create_alert_with_params(alert_params)
              logs << "\tPlatform alert already created. Trying to update.\n"
              logs << "\tFAIL: Update Platform alert\n" unless update_alert_with_params(alert_params)
            end
          end                  
        else
          # Steps when Druid Historical is OK
          stop_alert_with_params(alert_params)
        end

        logs
      end

      def show_errors(details)
        output = "Details:\n"
        details.each do |key, value|
          unless value.empty?
            output << "\t#{I18n.t("lib_redborder.errors_druid.#{key}")}\n"
            output << "\tError: #{value}\n\n"
          end
        end
        output
      end     

    end
  end
end
